local MAJOR, MINOR = "Xii_LibColor-1.0", 1
local lib = LibStub:NewLibrary(MAJOR, MINOR)

if not lib then
  return	-- already loaded and no upgrade necessary
end

local tonumber, table_insert, unpack, string_gmatch, string_len, string_sub, string_format, ipairs, setmetatable = 
	  tonumber, table.insert, unpack, string.gmatch, string.len, string.sub, string.format, ipairs, setmetatable


local COLOR_ALPHA_FIRST = 1
local COLOR_ALPHA_LAST = -1

local COLOR_FORMAT_RGBA = "rgba"
local COLOR_FORMAT_DECA = "deca"
local COLOR_FORMAT_HEXA = "hexa"
local COLOR_FORMAT_AHEX = "ahex"

lib.COLOR_ALPHA_FIRST = COLOR_ALPHA_FIRST
lib.COLOR_ALPHA_LAST  = COLOR_ALPHA_LAST
lib.COLOR_FORMAT_RGBA = COLOR_FORMAT_RGBA
lib.COLOR_FORMAT_DECA = COLOR_FORMAT_DECA
lib.COLOR_FORMAT_HEXA = COLOR_FORMAT_HEXA
lib.COLOR_FORMAT_AHEX = COLOR_FORMAT_AHEX

lib.__index = lib


function lib.hex2dec(hex)
	local result = {}

	for v in string_gmatch(hex, "(%x%x)") do
		table_insert(result, tonumber(v,16)/255)
	end
	
	return unpack(result)
end


function lib.hex2rgb(hex)
	local result = {}

	for v in string_gmatch(hex, "(%x%x)") do
		table_insert(result, tonumber(v,16))
	end

	return unpack(result)
end


function lib.hexFlipAlpha(hex, alphaPosition) -- 1 = first, -1 = last
	alphaPosition = alphaPosition or COLOR_ALPHA_FIRST

	if (string_len(hex) < 8) then
		return hex
	else
		local alpha, others

		if (alphaPosition == COLOR_ALPHA_LAST) then
			return string_sub(hex, -2) .. string_sub(hex, 1, -3)
		else
			return string_sub(hex, 3) .. string_sub(hex, 1, 2)
		end
	end
end


function lib.dec2hex(...)
	local result = ""

	for _,v in ipairs{...} do
		result = result .. string_format("%02x", v*255)
	end

	return result
end


function lib.rgb2hex(...)
	local result = ""

	for _,v in ipairs{...} do
		result = result .. string_format("%02x", v)
	end

	return result
end


function lib.dec2rgb(...)
	local result = {}

	for _,v in ipairs{...} do
		table_insert(result, v*255)
	end

	return unpack(result)
end


function lib.rgb2dec(...)
	local result = {}

	for _,v in ipairs{...} do
		table_insert(result, v/255)
	end

	return unpack(result)
end




function lib:New(r,g,b,a)
	local instance = {
		r = r or 255,
		g = g or 255,
		b = b or 255,
		a = a or 255,
		enabled = true,
	}

	return setmetatable(instance, lib)
end


function lib:enable()
	self.enabled = true
end


function lib:disable()
	self.enabled = false
end


function lib:IsEnabled()
	return self.enabled
end


function lib:SetEnabled(bool)
	self.enabled = bool
end


function lib:CreateFromRGBA(...)
	return self:New(...)
end


function lib:CreateFromDECA(...)
	return self:New(self.dec2rgb(...))
end


function lib:CreateFromAHEX(hex)
	return self:New(self.hex2rgb(self.hexFlipAlpha(hex, COLOR_ALPHA_FIRST)))
end


function lib:CreateFromHEXA(hex)
	return self:New(self.hex2rgb(hex))
end


function lib:SetRGBA(r, g, b, a)
	self.r = r or 255
	self.g = g or 255
	self.b = b or 255
	self.a = a or 255
end


function lib:SetDECA(...)
	self:SetRGBA(self.dec2rgb(...))
end


function lib:SetHEXA(...)
	self:SetRGBA(self.hex2rgb(...))
end


function lib:Format(format)
	if (format == COLOR_FORMAT_RGBA) then
		return self.r, self.g, self.b, self.a

	elseif (format == COLOR_FORMAT_DECA) then
		return self.rgb2dec(self.r, self.g, self.b, self.a)

	elseif (format == COLOR_FORMAT_HEXA) then
		return self.rgb2hex(self.r, self.g, self.b, self.a)

	elseif (format == COLOR_FORMAT_AHEX) then
		return self.rgb2hex(self.a, self.r, self.g, self.b)

	else
		print("ERROR: Unknown format specified", format)
		return false
	end
end


function lib:ToAHEX()
	return self:Format(COLOR_FORMAT_AHEX)
end


function lib:ToHEXA()
	return self:Format(COLOR_FORMAT_HEXA)
end


function lib:ToDECA()
	return self:Format(COLOR_FORMAT_DECA)
end


function lib:ToRGBA()
	return self:Format(COLOR_FORMAT_RGBA)
end
